@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2006, 2016, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2006, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -130,6 +137,61 @@ public:
     return 0;
   }
 
+  /**
+    @param query Query to execute
+    @return -1 if error, 1 if equal to value, 0 if different from value
+  */
+  int execute_conditional_query(const char* query, const char* value_to_compare)
+  {
+    int condition_result= -1;
+    if (!mysql_query(this->m_mysql_connection, query))
+    {
+      MYSQL_RES *result = mysql_store_result(this->m_mysql_connection);
+      if (result)
+      {
+        MYSQL_ROW row;
+        if ((row = mysql_fetch_row(result)))
+        {
+          condition_result= (strcmp(row[0], value_to_compare) == 0);
+        }
+        mysql_free_result(result);
+      }
+    }
+    return condition_result;
+  }
+
+  /**
+    @return -1 if error, 1 if user is not there, 0 if it is
+  */
+  int check_session_user_absence()
+  {
+    int no_session_user =
+      execute_conditional_query(
+        "SELECT COUNT(*) FROM mysql.user WHERE user = 'mysql.session'",
+        "0");
+    return no_session_user;
+  }
+
+  /**
+    @return -1 if error, 1 if the user is correctly configured, 0 if not
+  */
+  int check_session_user_configuration()
+  {
+    int is_user_configured = 0;
+    is_user_configured =
+      execute_conditional_query(
+        "SELECT SUM(count)=3 FROM ( "
+          "SELECT COUNT(*) as count FROM mysql.tables_priv WHERE "
+          "Table_priv='Select' and User='mysql.session' and Db='mysql' and Table_name='user' "
+          "UNION ALL "
+          "SELECT COUNT(*) as count FROM mysql.db WHERE "
+          "Select_priv='Y' and User='mysql.session' and Db='performance_schema' "
+          "UNION ALL "
+          "SELECT COUNT(*) as count FROM mysql.user WHERE "
+          "Super_priv='Y' and User='mysql.session') as user_priv;",
+        "1");
+    return is_user_configured;
+  }
   /**
     Error codes:
     EXIT_INIT_ERROR - Initialization error.
@@ -191,6 +253,35 @@ public:
     if (this->m_check_version && this->is_version_matching() == false)
       return EXIT_BAD_VERSION;
 
+    /*
+      Check and see if the Server Session Service default user exists
+    */
+
+    int user_is_not_there= check_session_user_absence();
+    int is_user_correctly_configured= 1;
+
+    if(!user_is_not_there)
+    {
+      is_user_correctly_configured= check_session_user_configuration();
+    }
+
+    if(!is_user_correctly_configured)
+    {
+      return this->print_error(
+          EXIT_UPGRADING_QUERIES_ERROR,
+          "The mysql.session exists but is not correctly configured."
+            " The mysql.session needs SELECT privileges in the"
+            " performance_schema database and the mysql.db table and also"
+            " SUPER privileges.");
+    }
+
+    if (user_is_not_there < 0 || is_user_correctly_configured < 0)
+    {
+      return this->print_error(EXIT_UPGRADING_QUERIES_ERROR,
+                               "Query against mysql.user table failed "
+                                 "when checking the mysql.session.");
+    }
+
     /*
       Run "mysql_fix_privilege_tables.sql" and "mysqlcheck".
 
